# Generated by roxygen2: do not edit by hand

S3method(plot,gen_sobol)
S3method(plot,var_decomp)
S3method(predict,onam)
S3method(print,gen_sobol)
S3method(print,summary.onam)
S3method(print,var_decomp)
S3method(summary,onam)
export(build_dnn)
export(decompose)
export(gen_sobol)
export(install_conda_env)
export(load_onam)
export(onam)
export(plot_inter_effect)
export(plot_main_effect)
export(save_onam)
importFrom(dplyr,"%>%")
importFrom(keras3,fit)
importFrom(reticulate,py_available)
importFrom(rlang,.data)
importFrom(stats,predict)
