% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_decomposition.R
\name{plot.gen_sobol}
\alias{plot.gen_sobol}
\title{Plot generalized Sobol indices}
\usage{
\method{plot}{gen_sobol}(x, ...)
}
\arguments{
\item{x}{Object of class \code{gen_sobol} as returned by \link{gen_sobol}}

\item{...}{further arguments, currently unused}
}
\value{
Returns a 'ggplot2' object showing, for each fitted effect, the
generalized Sobol index split into the effect's own contribution
(\code{gen_sobol_index_1}) and its contribution through interactions with
other effects of the same order (\code{gen_sobol_index_2})
}
\description{
Plot generalized Sobol indices
}
\details{
For details on generalized sobol indices, see
Chastaing et al. (2012) \doi{10.1214/12-EJS749}.
}
