% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/persistence.R
\name{load_onam}
\alias{load_onam}
\title{Load a fitted onam model from disk}
\usage{
load_onam(dir)
}
\arguments{
\item{dir}{Directory as created by \link{save_onam}.}
}
\value{
Returns the restored object of class \code{onam}.
}
\description{
Load a fitted onam model from disk
}
\seealso{
\link{save_onam}
}
