% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_decomposition.R
\name{plot.var_decomp}
\alias{plot.var_decomp}
\title{Plot variance decomposition}
\usage{
\method{plot}{var_decomp}(x, ...)
}
\arguments{
\item{x}{Object of class \code{var_decomp} as returned by \link{decompose}}

\item{...}{further arguments, currently unused}
}
\value{
Returns a 'ggplot2' object showing the fraction of total
variance explained by each interaction order
}
\description{
Plot variance decomposition
}
